#!/usr/bin/python3

"""
    TLBE - 'table lookup, to bytes' encryption
    Creative Commons Zero 1.0 Universal
"""

import random
import base64
import numpy as np

chunksplit = lambda l, d: [l[i : i + d] for i in range(0, len(l), d)]
to_base64 = lambda text: base64.b64encode(text.encode()).decode()
from_base64 = lambda text: base64.b64decode(text.encode()).decode()


class TLBE:
    def take_from_table(self, notation, table):
        raise NotImplemented

    def take_random_pos_of(self, finding_char):
        raise NotImplemented
        if not finding_char in self.alphabet:
            _ = "The character of order {0:d} is not included in the table/alphabet"
            _ = IndexError(_.format(ord(finding_char)))
            raise (_)

        res = []
        for index, layer2 in table:
            for index2, char in layer2:
                if char == finding_char:
                    res.append([index, index2])
        return res

    def generate_table(self):
        """
        Remarks:
        * In theory, it should replace characters in the table that is more
        than once with the one that is missing... until either all characters
        are missing, or the capacity of the 16x16 table is exhausted.
        * If there's not all 256 characters that fill the all 256 cells, and
        the message is short enough, than chances are the table still can not
        be bruteforced.
        """
        collect = []
        for i in range(16):
            res = []
            for i in range(16):
                res.append(random.choice(self.alphabet))
            collect.append(res)
        real_size = 0
        collected_symbols = []
        for i, j in enumerate(collect):
            for k, l in enumerate(j):
                if l in self.alphabet and not l in collected_symbols:
                    real_size += 1
                    collected_symbols.append(l)
        if real_size == self.alphabet_size:
            return collect
        else:
            # formed = "The table was generated for {0:d} of {1:d} characters"
            # formed = formed.format(real_size, self.alphabet_size)
            # print(formed)

            self.present_characters = collected_symbols
            return False  # Hotfix: "Need to regenerate"

    def display(self):
        res = (
            " " * 4
            + " ".join(["{:X}".format(i) for i in range(16)])
            + "\n"
            + " " * 2
            + "+"
            + "-" * 16 * 2
            + "\n"
        )
        for i1, j1 in enumerate(self.table):
            collected_line = []
            for j2 in j1:
                if ord(j2) == 10 or ord(j2) == 13:
                    j2 = "."
                collected_line.append(j2)
            res += "{0:X} | {1:s}\n".format(i1, " ".join(collected_line))
        print(res)

    def encode(self, text):
        res = ""
        for index, char in enumerate(text):
            options_for_char = []
            for height_pos, horizontal in enumerate(self.table):
                for width_pos, table_char in enumerate(horizontal):
                    if table_char == char:
                        wp = hex(width_pos)[2:]
                        hp = hex(height_pos)[2:]
                        options_for_char.append("{:s}{:s}".format(wp, hp))
            try:
                used_option = random.choice(options_for_char)
                res += used_option
            except IndexError as e:
                _ = "The character of order {0:d} is not included in the table"
                _ = _.format(ord(char))
                print(_)
        return res

    def decode(self, ciphertext):
        res = ""
        ciphertext = chunksplit(ciphertext, 2)
        for index, chunk in enumerate(ciphertext):
            try:
                width_pos, height_pos = int(chunk[0], 16), int(chunk[1], 16)
            except IndexError:
                _ = IndexError("Incorrect padding.")
                _.__cause__ = None
                raise _
            except ValueError:
                _ = ValueError("Incorrect format.")
                _.__cause__ = None
                raise _
            selected = self.table[height_pos][width_pos]
            res += selected
        return res

    def generate(self):
        gen_table = self.generate_table()
        if not gen_table is False:
            self.table = gen_table
        elif gen_table is False:
            # Let's dedicate space then!
            setA = set(self.alphabet)
            setB = set(self.present_characters)
            should_move_in = setA - setB
            should_move_in = list(should_move_in)
            random.shuffle(should_move_in)

            counts = {}
            counter = 0  # Not using .pop(0) for array because not all
            #              characters can be covered on the second check below.
            for row_index, row in enumerate(self.table):
                for column_index, char in enumerate(row):
                    if not should_move_in:
                        break
                    if counts.get(char, -1) > 1 and counter < len(should_move_in):
                        going_char = should_move_in[counter]
                        self.table[row_index][column_index] = going_char
                        counter += 1
                    counts[char] = counts.get(char, 0) + 1

            # Should check here again if using UTF-16!
            have_got = []
            left_to_check = len(should_move_in)
            for row_index, row in enumerate(self.table):
                for column_index, char in enumerate(row):
                    is_last_cell = row_index == 15 and column_index == 15
                    if char in should_move_in and not char in have_got:
                        left_to_check -= 1
                        have_got.append(char)
                    elif left_to_check > 0 and is_last_cell:
                        left_to_check = -1

            if left_to_check == 0:
                return

            # Least case ↓ e.g. when using UTF-16.
            faultlog = "Missing characters:\n"
            missing = should_move_in
            missing = ", ".join([str(ord(c)) for c in missing])
            faultlog += "%s" % missing
            open("TLBE.log", "w").write("%s" % faultlog)
            raise KeyError(
                "Not enough characters in the table. "
                + "Missing characters information is saved in the TLBE.log file."
            )

    def calculate_alphabet_size(self, alphabet):
        # self.alphabet_size = len(self.alphabet)
        alphabet_size = len(alphabet)
        collected_symbols = []
        for j in alphabet:
            if j in collected_symbols:
                alphabet_size -= 1
            else:
                collected_symbols.append(j)
        return alphabet_size

    def __init__(self, alphabet_grade):
        if alphabet_grade > 4:
            alphabet = [chr(i) for i in range(0, 256)]
            self.table = alphabet
            random.shuffle(self.table)
            self.table = chunksplit(np.array(self.table), 16)
        else:
            alphabet = [chr(i) for i in range(ord("A"), ord("Z") + 1)]
            if alphabet_grade > 1:
                alphabet += [c.lower() for c in alphabet]
            if alphabet_grade > 2:
                alphabet += list(""" '".,:;!?-/""")
            if alphabet_grade > 3:
                alphabet += [chr(i) for i in range(ord("0"), ord("9") + 1)]
        self.alphabet = alphabet
        if alphabet_grade <= 4:
            self.alphabet_size = self.calculate_alphabet_size(alphabet)


enlinear = lambda t: list(np.array(t).reshape([1, 16**2])[0])
ensort = lambda t: list(np.array(list(t)).reshape([16, 16]))


def save_table(filepath, table):
    res = ""
    res = enlinear(table)
    res = to_base64("".join(res))
    with open(filepath, "w") as f:
        out_size = f.write(res)
    return out_size


def load_table(filepath):
    with open(filepath, "r") as f:
        content = f.read()
    content = from_base64(content)
    content = ensort(content)
    return content


def convert_to_bytes(ciphertext):
    return ciphertext.encode()


def convert_from_bytes(ciphertext):
    return ciphertext.decode()


proc_user_input = lambda: input("Choice: ").strip()[0]


def interactive_mode():
    print(
        """\
1. Load table
2. Generate a random table and save it"""
    )
    choice = proc_user_input()
    our_instance = TLBE(4)
    if choice == "1":
        print("File path:")
        our_instance.table = load_table(input())
        print("Loaded")
    elif choice == "2":
        our_instance.generate()
        print("File path:")
        try:
            inp = input()
            if not inp.strip():
                raise BaseException
            save_table(inp, our_instance.table)
        except KeyboardInterrupt or EOFError or FileNotFoundError or BaseException:
            print("Not saving? Then write down the table:")
            our_instance.display()
            input()
        print("Randomized")
    else:
        raise SystemExit
    while True:
        res = None
        print(
            """
1. Encode text
2. Decode text
3. Load table
4. Save table
5. Display the table
6. Reshuffle (generate it again)
0. Quit
        """.strip()
        )
        try:
            choice = proc_user_input()
        except IndexError:
            break
        if choice == "1":
            print("Text:")
            source_text = input()
            res = our_instance.encode(source_text)
        elif choice == "2":
            print("Ciphertext:")
            source_ciphertext = input()
            res = our_instance.decode(source_ciphertext)
        elif choice == "3":
            print("File path:")
            fp = input()
            our_instance.table = load_table(fp)
            print("Loaded")
        elif choice == "4":
            print("File path:")
            fp = input()
            save_table(fp, our_instance.table)
            print("Saved")
        elif choice == "5":
            res = our_instance.display()
        elif choice == "6":
            our_instance.generate()
            print("Reshuffled")
        elif choice == "0":
            break
        if isinstance(res, str) and res:
            print("Result:\n" + res)
        input()
    print("Goodbye!")
    raise SystemExit


if __name__ == "__main__":
    DEBUG = True
    import sys
    import re

    argv = sys.argv
    argc = len(argv)
    aq = " ".join(argv[1:])

    print("TLBE v0.7")

    paramz = [
        "interactive"
        if DEBUG or re.match(r"^-([Ii]|-interactive(-mode)?)", aq)
        else None
    ]

    if "interactive" in paramz:
        try:
            res = interactive_mode()
            raise SystemExit(res)
        except KeyboardInterrupt or EOFError:
            raise SystemExit

    print("Huh.")
