#!/usr/bin/python3

from os.path import basename
from sys import argv
import numpy as np
import wave

def main(argv: list) -> None:
	argc = len(argv)
	show_help = argc < 2 or argc > 3
	show_help = show_help or (argc >= 2 and argv[1] in ["-h", "--help"])
	if show_help:
		print(f"Usage: {basename(argv[0])} <input file name> "
			"[output file name]")
		print("< >: Positional arguments")
		print("[ ]: Optional arguments")
		return

	input_file = argv[1]
	if argc == 2:
		output_file = input_file
	elif argc == 3:
		output_file = argv[2]

	input_wave = wave.open(input_file, "rb")
	nchannels, sampwidth, framerate, nframes = input_wave.getparams()[:4]

	if sampwidth == 2:
		np_type = np.int16
	else:
		print("Unsupported bit depth:", 8 * sampwidth)
		return

	audio_data = input_wave.readframes(nframes)

	audio_array = np.frombuffer(audio_data, dtype=np_type).byteswap()

	output_wave = wave.open(output_file, "wb")
	output_wave.setparams(
		(nchannels, sampwidth, framerate, 0, "NONE", "not compressed"))
	output_wave.writeframes(audio_array.tobytes())
	output_wave.close()

	print("Done")

if __name__ == "__main__":
	main(argv)

