#!/usr/bin/python3

from timeit import timeit
import random

BENCHMARK_COUNT = 900
SUPERTESTS = 16
MAX_SHIFT_VALUE = 18

op_and = lambda a, b: a & (b - 1)
op_mod = lambda a, b: a % b

timeit_macro = lambda func_name, number, x: timeit(
    f"%s(2**15, {x})" % func_name,
    setup="from __main__ import %s\nimport random" % func_name,
    number=number,
)

if __name__ == "__main__":
    print(
        f"Testing performance of modulo and bitwise AND remainder methods for binary divisors from 1 to {1 << (SUPERTESTS - 1)}:"
    )

    time1_total = 0
    time2_total = 0
    number = 1
    for i in range(SUPERTESTS):
        bench_now = BENCHMARK_COUNT * i
        time1 = timeit_macro("op_and", bench_now, number)
        time2 = timeit_macro("op_mod", bench_now, number)
        time1_total += time1
        time2_total += time2
        number <<= 1

    time1_avg = time1_total / SUPERTESTS
    time2_avg = time2_total / SUPERTESTS

    print(f"Method #1 (AND operator):\t{time1_avg:.16f} seconds")
    print(f"Method #2 (MOD operator):\t{time2_avg:.16f} seconds")

    if time1 < time2:
        print("Method #1 is faster")
    elif time1 > time2:
        print("Method #2 is faster")
    else:
        print("Methods have similar performance")
