#!/usr/bin/python3

from timeit import timeit
import math

BENCHMARK_COUNT = 256

op_floor_div = lambda a: math.floor(a / 64)
op_int_div = lambda a: a // 64
op_bw_div = lambda a: (a >> 6) << 6

timeit_macro = lambda func_name: timeit(
    f"%s(250)" % func_name,
    setup="from __main__ import %s\nimport math" % func_name,
    number=BENCHMARK_COUNT,
)

if __name__ == "__main__":
    print(
        f"Testing performance of math.floor(250 / 64) vs. 250 // 64 vs. (250 >> 6) << 6:"
    )

    time1 = timeit_macro("op_floor_div")
    time2 = timeit_macro("op_int_div")
    time3 = timeit_macro("op_bw_div")

    print(f"Method #1 (floor division):\t{time1} seconds")
    print(f"Method #2 (integer division):\t{time2} seconds")
    print(f"Method #3 (bitwise division):\t{time3} seconds")

    if time1 < time2 and time1 < time3:
        print("Method #1 is faster")
    elif time2 < time1 and time2 < time3:
        print("Method #2 is faster")
    elif time3 < time1 and time3 < time2:
        print("Method #3 is faster")
    else:
        print("Methods have similar performance")
