#!/usr/bin/python3

from timeit import timeit

BENCHMARK_COUNT = 90
SUPERTESTS = 18


def lower_method1():
    for x in range(0, 256):
        chr(x | 0x20)


def lower_method2():
    for x in range(0, 256):
        chr(x).lower()


timeit_macro = lambda func_name, number: timeit(
    "%s()" % func_name, setup="from __main__ import %s" % func_name, number=number
)

if __name__ == "__main__":
    print("Testing performance of two string lowering methods:")

    time1_total = 0
    time2_total = 0
    for i in range(SUPERTESTS):
        bench_now = BENCHMARK_COUNT * i
        time1 = timeit_macro("lower_method1", bench_now)
        time2 = timeit_macro("lower_method2", bench_now)
        time1_total += time1
        time2_total += time2

    time1_avg = time1_total / SUPERTESTS
    time2_avg = time2_total / SUPERTESTS

    print(f"Method #1 (bitwise):\t{time1_avg} seconds")
    print(f"Method #2 (.lower()):\t{time2_avg} seconds")

    if time1 < time2:
        print("Method #1 is faster")
    elif time1 > time2:
        print("Method #2 is faster")
    else:
        print("Methods have similar performance")
