#!/usr/bin/python3

import midiutil
import os
import random
import sys

NOTE_TABLE = {i: 52 + i for i in range(8 * 2)}

is_prime = lambda n: n > 1 and all(n % i for i in range(2, n))

TEMPO = 90
TIME_SIGNATURE = (4, 4)
INSTRUMENT_SELECTED = 8
STEP_SIZE = 16
duration_constant = 2


def binary_to_midi(input_file, output_file):
    print("Reading file")
    with open(input_file, "rb") as f:
        bytes = f.read()

    print("Initialization")
    pattern = midiutil.MIDIFile(1, TIME_SIGNATURE, TEMPO)
    pattern.addTempo(0, 0, TEMPO)
    track = 0
    channel = 0
    time = 0

    chords = []
    current_chord = []

    previous_note = NOTE_TABLE[0]
    byte_step = (len(bytes) // 2**6) + 1

    print("Converting")
    taking_chord = False
    for i, b in enumerate(bytes[::STEP_SIZE]):
        is_rest = False
        note = NOTE_TABLE[b % len(NOTE_TABLE)]
        is_previous_note = i > 0 and note == previous_note

        if b % 51 == 25:
            is_rest = True
        elif b % 12 == 0:
            current_chord = [note]
            chords.append(current_chord)
            taking_chord = True
        elif taking_chord:
            current_chord.append(note)
            taking_chord = False

        duration = (0.25 if is_prime(b) or b % 11 < 6 else 0.015) * duration_constant

        if abs(previous_note - note) > 3 and not is_previous_note:
            current_chord.append(0)
            previous_note = note

    pattern.addProgramChange(0, 0, 0, INSTRUMENT_SELECTED)
    for chord in chords:
        for note in chord:
            if not is_rest:
                pattern.addNote(track, channel, note, time, duration, 100)
            else:
                pattern.addNote(track, channel, 0, time, duration, 0)
        time += duration

    print("Writing out")
    with open(output_file, "wb") as outf:
        pattern.writeFile(outf)
    file_size = os.path.getsize(output_file)
    print(f"Saved, {file_size} bytes")


if __name__ == "__main__":
    if len(sys.argv) > 1:
        binary_to_midi(sys.argv[1], "output.mid")
    else:
        print("Fatal error: No file specified as argument.")
