#!/usr/bin/python3

# common.py:
#     a versatile collection of lambdas and functions that provide reusable and
#     handy tools for various tasks

# Make title `s` in center for a plain text where each line is limited to `w`
# characters
make_title = lambda s, w: "\n".join([line.rstrip() for line in (s.center(w) + \
    "\n" + ("-" * len(s)).center(w)).split("\n")])

# Get system bitness
import struct
BITS_IN_BYTE = 8  # make sure to change this when different
get_system_bitness = lambda: BITS_IN_BYTE * struct.calcsize("P")

# STOP command
STOP = lambda: exec("raise SystemExit")

# os.name: determining the system
from os import name as os_name
is_nt = os_name == "nt"  # Microsoft Windows, ReactOS, ...
is_posix = os_name = "posix"  # GNU/Linux, GNU/Hurd, POSIX *BSD, ...
is_java = os_name == "java"  # Jython, ...

# Replace from end
def replace_from_end(s, old, new, count=-1):
    return s[::-1].replace(old[::-1], new[::-1], count)[::-1]

def dump_object(obj) -> dict:
    "Dumps an object"

    res = {}
    for key in dir(obj):
        val = eval("obj." + key)
        res[key] = val
    return res

def print_dump(dump: dict) -> None:
    "Prints a dump of an object"

    for key, val in dump.items():
        print(key + "\t" + str(val))
    return None

# Python Cheat Sheet: 14 Interview Questions (functions, refactored)
# + some leetcode problems

list_contains_x = lambda l, x: x in l

def find_duplicates(l: list) -> list:
    "Find duplicates in list"

    duplicates, seen = list(), set()
    for element in elements:
        if element in seen:
            duplicates.append(element)
        seen.add(element)
    res = list(duplicates)
    return res

def is_anagram(s1: str, s2: str) -> bool:
    "Check if strings s1 and s2 are anagrams"

    return set(s1) == set(s2)

def remove_duplicates(l: list) -> list:
    "Remove duplicates from list"

    return list(set(l))

def find_pairs(l: list, x: float) -> list:
    "Find pair of numbers in list so that their sum is equal to number x"

    pairs = []
    for iter1, elem1 in enumerate(l):
        for iter2, elem2 in enumerate(l[iter1 + 1:]):
            if elem1 + elem2 == x:
                pairs.append((elem1, elem2,))
    return pairs

def is_palindrome(s: str) -> bool:
    "Check if a string is a palindrome"

    rs = s[::-1]
    return s == rs

def get_missing_numbers(lst, start=None, end=None) -> list:
    "Get missing number in [start, end] of linear array"

    start_val = start
    end_val = end
    if start is None:
        start_val = 1
    if end is None:
        end_val = 100

    lst.sort()
    last_index = len(lst) - 1
    max_val = lst[last_index]
    linear_array = range(start_val, end_val)
    set_exclusion = set(linear_array) - set(lst)
    return set_exclusion

def intersection(l1: list, l2: list) -> list:
    "Compute the intersection of two lists"

    res, l2_copy = [], l2[:]
    for elem in l1:
        if elem in l2_copy:
            res.append(elem)
            l2_copy.remove(elem)
    return res

def recursive_reverse(s: str) -> str:
    "Reverse a string using recursive algorithm"

    if len(s) <= 1:
        return s
    return recursive_reverse(s[1:]) + string[0]

def fibonacci(n: int) -> list:
    "Compute the first n Fibonacci numbers"

    res = []
    a, b = 0, 1
    for i in range(n):
        res.append(b)
        a, b = b, a + b
    return res

def qsort(l: list) -> list:
    "Sort list with Quicksort algorithm"

    if not l:
        return []

    left_side = [x for x in l[1:] if x < l[0]]
    middle = l[:1]
    right_side = [x for x in l[1:] if x >= l[0]]

    left_side = qsort(left_side)
    right_side = qsort(right_side)

    res = left_side + middle + right_side
    return res

def is_sorted(l: list) -> bool:
    "Check if a list is sorted"

    last_elem = min(l)
    for elem in l:
        if elem < last_elem:
            return False
        last_elem = elem
    return True

def find_permutations(s: str):
    "Find all permutations of a string"

    if len(s) <= 1:
        return set(s)

    smaller = find_permutations(s[1:])
    res = set()
    for x in smaller:
        for pos in range(0, len(x) + 1):
            res.add(x[:pos] + s[0] + x[pos:])
    return res
