#!/usr/bin/python3

import os
import psutil
import platform

if platform.system() != "Windows":
    print("This script is for Microsoft Windows OSes only")
    raise SystemExit

dirs_to_scan = ["SYSTEMROOT", "programfiles", "programfiles(x86)"]
column_limit = 80
scan_prefix = "scanning: "
shortening_prefix = "..."
scan_prefix_length = len(scan_prefix)
shortening_prefix_len = len(shortening_prefix)
cut_condition_value = column_limit - 1 - scan_prefix_length
left_cut_offset = cut_condition_value - shortening_prefix_len
wrap_string = (
    lambda s: shortening_prefix + s[-left_cut_offset:]
    if len(s) > cut_condition_value
    else s
)


def get_directory_size(directory):
    total_size = 0
    for path, dirs, files in os.walk(directory):
        print(scan_prefix + wrap_string(path).replace("/", "\\"), end="")
        for file in files:
            file_path = os.path.join(path, file)
            total_size += os.path.getsize(file_path)
        print("\r", end="")
    print(" " * column_limit, end="")
    return total_size


def display_sizes(size, partition_size=None):
    print(size, "bytes =")
    print("%.2f kibibytes =" % (size / 1024**1))
    print("%.2f mebibytes =" % (size / 1024**2))
    print("%.2f gibibytes" % (size / 1024**3))
    if not partition_size is None:
        this_percentage = size / partition_size * 100
        print("That's %.2f%% of the partition" % this_percentage)


if __name__ == "__main__":
    print("Trying to get environmental variables...")
    new_dirs_to_scan = []
    for dir in dirs_to_scan:
        res = os.environ.get(dir)
        if not res is None:
            new_dirs_to_scan.append(res)
    dirs_to_scan = new_dirs_to_scan

    print("\nDirectories to sum:")
    for elem in dirs_to_scan:
        print("-", elem)
    print()

    try:
        res = input('Press Enter to continue or Ctrl+C / type "c" to cancel: ')
        if res and res.strip().lower()[0] == "c":
            raise SystemExit
    except KeyboardInterrupt or EOFError:
        raise SystemExit

    print("-" * column_limit)

    total_size = 0

    # assume that those directories are on the same partition
    partition_size = psutil.disk_usage(
        dirs_to_scan[0].split("\\")[0]
    ).total

    for elem in dirs_to_scan:
        print(elem)
        directory_path = elem.replace("\\", "/")

        print("Calculating its size")
        this_size = get_directory_size(directory_path)
        print("Total file size of directory %s:" % directory_path)
        display_sizes(this_size, partition_size)
        print()
        total_size += this_size

    print("-" * column_limit)
    print("Those directories take in total:")
    display_sizes(total_size, partition_size)
